<?php

namespace Drupal\commerce_order2pdf\Controller;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Utility\Token;
use Mpdf\Mpdf;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

/**
 * Provides PDF generation functionality for commerce orders.
 */
class Order2pdfController extends ControllerBase {

  use StringTranslationTrait;

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * The commerce_order2pdf settings configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a new Order2pdfController object.
   *
   * @param \Drupal\Core\Utility\Token $token
   *   The token service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(Token $token, ConfigFactoryInterface $config_factory, RendererInterface $renderer, RequestStack $requestStack, MessengerInterface $messenger, FileSystemInterface $file_system) {
    $this->token = $token;
    $this->config = $config_factory->get('commerce_order2pdf.settings');
    $this->renderer = $renderer;
    $this->requestStack = $requestStack;
    $this->messenger = $messenger;
    $this->fileSystem = $file_system;
    $this->siteName = $config_factory->get('system.site')->get('name');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('token'),
      $container->get('config.factory'),
      $container->get('renderer'),
      $container->get('request_stack'),
      $container->get('messenger'),
      $container->get('file_system'),
    );
  }

  /**
   * Displays the PDF view for the given commerce order.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $commerce_order
   *   The commerce order.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The PDF response.
   */
  public function view(OrderInterface $commerce_order) {
    // Delegate to the function handling the logic.
    return $this->commerceOrder2pdfView($commerce_order);
  }

  /**
   * Handles access control for PDF generation.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $commerce_order
   *   The commerce order.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(OrderInterface $commerce_order) {
    // Delegate to the function handling the access check.
    return $this->commerceOrder2pdfAccess($commerce_order, 'validate');
  }

  /**
   * Provides PDF generation functionality for commerce orders.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $commerce_order
   *   The commerce order.
   */
  public function commerceOrder2pdfView(OrderInterface $commerce_order) {

    // Get the value of file_temp_path from settings.php.
    $file_temp_path = Settings::get('file_temp_path');

    if (!empty($file_temp_path)) {
      $mpdf = new Mpdf(['tempDir' => $file_temp_path]);
    }
    else {
      $temp_directory = $this->fileSystem->getTempDirectory();
      // Ensure that the directory exists, create it if necessary.
      $mpdf = new Mpdf(['tempDir' => $temp_directory]);
    }
    // $html must be defined
    $token = $this->token;
    $config = $this->config;
    $style_file = $config->get('pdf_style_file');
    $style = file_get_contents($style_file);

    $pdf_header = $config->get('pdf_header');
    $replacements = ['commerce-order' => $commerce_order];
    $html_header = $token->replace($pdf_header, $replacements);
    $commerce_id = $commerce_order->id();
    $view = views_embed_view('commerce_order2pdf', 'default', $commerce_id);
    $html = $html_header . $this->renderer->render($view);

    $mpdf->WriteHTML($style, 1);
    $mpdf->WriteHTML($html);
    $site_name = $this->siteName;
    $pdf_title = $site_name . '_' . $config->get('pdf_title');

    $replacements = ['commerce-order' => $commerce_order];
    $pdf_title_replaced = $token->replace($pdf_title, $replacements);
    $pdf_filename = $pdf_title_replaced . ".pdf";
    $mpdf->Output($pdf_filename, 'd');
    // Get the PDF content as a string.
    $pdf_content = $mpdf->Output('', 'S');

    // Create a temporary file to save the PDF content.
    $temp_file = tempnam($temp_directory, $site_name . '_' . 'commerce_order_pdf_');
    file_put_contents($temp_file, $pdf_content);
    // Get the PDF filename.
    $pdf_filename = $site_name . '_' . 'commerce_order_' . $commerce_order->id() . '.pdf';
    // Create a BinaryFileResponse pointing to the temporary file.
    $response = new BinaryFileResponse($temp_file);
    $response->setContentDisposition(
        ResponseHeaderBag::DISPOSITION_ATTACHMENT,
        $pdf_filename
    );
    return $response;
  }

  /**
   * Access callback for PDF generation.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The commerce order.
   * @param string $action
   *   The action to perform.
   *
   * @return mixed
   *   The hash if action is 'create', or AccessResultInterface otherwise.
   */
  public function commerceOrder2pdfAccess(OrderInterface $order, $action = 'create') {

    // Get the current URL path.
    $current_request = $this->requestStack->getCurrentRequest();

    $arg2 = $current_request->attributes->get('hash') ?? NULL;

    $hash = hash('md5', $order->id() . $order->getCreatedTime());
    if ($action == 'create') {
      return $hash;
    }
    if ($action == 'validate') {
      if ($arg2 == $hash) {
        return AccessResult::allowed();
      }
      if ($arg2 != $hash) {
        $error_message = $this->t('Sorry, wrong access key. Please contact a site administrator!');
        $this->messenger()->addError($error_message);

        return AccessResult::forbidden();
      }
    }
  }

}
